@@ -1,4 +1,4 @@
-# Copyright (C) 2018-2021 all contributors <meta@public-inbox.org>
+# Copyright (C) all contributors <meta@public-inbox.org>
 # License: AGPL-3.0+ <https://www.gnu.org/licenses/agpl-3.0.txt>
 
 # for XOVER, OVER in NNTP, and feeds/homepage/threads in PSGI
@@ -17,10 +17,11 @@ use PublicInbox::MID qw/id_compress mids_for_index references/;
 use PublicInbox::Smsg qw(subject_normalized);
 use Compress::Zlib qw(compress);
 use Carp qw(croak);
+use bytes (); # length
 
 sub dbh_new {
 	my ($self) = @_;
-	my $dbh = $self->SUPER::dbh_new($self->{-no_fsync} ? 2 : 1);
+	my $dbh = $self->SUPER::dbh_new(1);
 
 	# 80000 pages (80MiB on SQLite <3.12.0, 320MiB on 3.12.0+)
 	# was found to be good in 2018 during the large LKML import
@@ -33,8 +34,9 @@ sub dbh_new {
 }
 
 sub new {
-	my ($class, $f) = @_;
+	my ($class, $f, $opt) = @_;
 	my $self = $class->SUPER::new($f);
+	$self->{-opt} = $opt;
 	$self->{min_tid} = 0;
 	$self;
 }
@@ -138,8 +140,7 @@ SELECT id FROM msgid WHERE mid = ? LIMIT 1
 	my $id = $sth->fetchrow_array;
 	defined $id or return;
 
-	push(@$cols, 'num');
-	$cols = join(',', map { $_ } @$cols);
+	$cols = join ',', @$cols, 'num';
 	my $lim = 10;
 	my $prev = get_counter($dbh, 'ghost');
 	while (1) {
@@ -199,7 +200,7 @@ sub resolve_mid_to_tid {
 	$tid // do { # create a new ghost
 		my $id = mid2id($self, $mid);
 		my $num = next_ghost_num($self);
-		$num < 0 or die "ghost num is non-negative: $num\n";
+		$num < 0 or croak "BUG: ghost num is non-negative: $num\n";
 		$tid = next_tid($self);
 		my $dbh = $self->{dbh};
 		$dbh->prepare_cached(<<'')->execute($num, $tid);
@@ -263,7 +264,10 @@ sub ddd_for ($) {
 
 sub add_overview {
 	my ($self, $eml, $smsg) = @_;
-	$smsg->{lines} = $eml->body_raw =~ tr!\n!\n!;
+	my $raw = $eml->body_raw;
+	$smsg->{lines} = $raw =~ tr!\n!\n!;
+	$smsg->{bytes} //= bytes::length $raw;
+	undef $raw;
 	my $mids = mids_for_index($eml);
 	my $refs = $smsg->parse_references($eml, $mids);
 	$mids->[0] //= do {
@@ -283,7 +287,7 @@ sub _add_over {
 	my ($self, $smsg, $mid, $refs, $old_tid, $v) = @_;
 	my $cur_tid = $smsg->{tid};
 	my $n = $smsg->{num};
-	die "num must not be zero for $mid" if !$n;
+	croak "BUG: num must not be zero for $mid" if !$n;
 	my $cur_valid = $cur_tid > $self->{min_tid};
 
 	if ($n > 0) { # regular mail
@@ -337,6 +341,7 @@ INSERT INTO id2num (id, num) VALUES (?,?)
 		my $id = mid2id($self, $mid);
 		$sth->execute($id, $num);
 	}
+	$self->{-art_max} = $num if $num > ($self->{-art_max} // 0);
 }
 
 sub _remove_oid {
@@ -385,7 +390,7 @@ CREATE TABLE IF NOT EXISTS over (
 	sid INTEGER, /* Subject ID (IMAP ORDEREDSUBJECT "threading") */
 	ts INTEGER, /* IMAP INTERNALDATE (Received: header, git commit time) */
 	ds INTEGER, /* RFC-2822 sent Date: header, git author time */
-	ddd VARBINARY /* doc-data-deflated (->to_doc_data, ->load_from_data) */
+	ddd BLOB /* doc-data-deflated (->to_doc_data, ->load_from_data) */
 )
 
 	$dbh->do('CREATE INDEX IF NOT EXISTS idx_tid ON over (tid)');
@@ -435,6 +440,7 @@ sub commit_lazy {
 	delete $self->{txn} or return;
 	$self->{dbh}->commit;
 	eval { $self->{dbh}->do('PRAGMA optimize') };
+	warn 'W: optimize ', $self->{dbh}->sqlite_db_filename, ': ', $@ if $@;
 }
 
 sub begin_lazy {
@@ -454,12 +460,12 @@ sub rollback_lazy {
 
 sub dbh_close {
 	my ($self) = @_;
-	die "in transaction" if $self->{txn};
+	Carp::confess('BUG: in transaction') if $self->{txn};
 	$self->SUPER::dbh_close;
 }
 
 sub create {
-	my ($self) = @_;
+	my ($self, $opt) = @_;
 	my $fn = $self->{filename} // do {
 		croak('BUG: no {filename}') unless $self->{dbh};
 		return;
@@ -509,18 +515,18 @@ EOF
 				next;
 			}
 			$pr->(<<EOM) if $pr;
-I: ghost $r->{num} <$mid> THREADID=$r->{tid} culled
+# ghost $r->{num} <$mid> THREADID=$r->{tid} culled
 EOM
 		}
 		delete_by_num($self, $r->{num});
 	}
-	$pr->("I: rethread culled $total ghosts\n") if $pr && $total;
+	$pr->("# rethread culled $total ghosts\n") if $pr && $total;
 }
 
 # used for cross-inbox search
 sub eidx_prep ($) {
 	my ($self) = @_;
-	$self->{-eidx_prep} //= do {
+	$self->{-eidx_prep} // do {
 		my $dbh = $self->dbh;
 		$dbh->do(<<'');
 INSERT OR IGNORE INTO counter (key) VALUES ('eidx_docid')
@@ -537,7 +543,7 @@ CREATE TABLE IF NOT EXISTS xref3 (
 	docid INTEGER NOT NULL, /* <=> over.num */
 	ibx_id INTEGER NOT NULL, /* <=> inboxes.ibx_id */
 	xnum INTEGER NOT NULL, /* NNTP article number in ibx */
-	oidbin VARBINARY NOT NULL, /* 20-byte SHA-1 or 32-byte SHA-256 */
+	oidbin BLOB NOT NULL, /* 20-byte SHA-1 or 32-byte SHA-256 */
 	UNIQUE (docid, ibx_id, xnum, oidbin)
 )
 
@@ -565,7 +571,7 @@ CREATE TABLE IF NOT EXISTS eidx_meta (
 		$dbh->do(<<'');
 CREATE TABLE IF NOT EXISTS eidxq (docid INTEGER PRIMARY KEY NOT NULL)
 
-		1;
+		$self->{-eidx_prep} = 1;
 	};
 }
 
@@ -670,4 +676,17 @@ sub vivify_xvmd {
 	$smsg->{-vivify_xvmd} = \@vivify_xvmd;
 }
 
+sub fork_ok {
+	state $fork_ok = eval("v$DBD::SQLite::sqlite_version") ge v3.8.3;
+	return 1 if $fork_ok;
+	my ($opt) = @_;
+	my @j = split(/,/, $opt->{jobs} // '');
+	state $warned;
+	grep { $_ > 1 } @j and $warned //= warn(<<EOM);
+DBD::SQLite version is v$DBD::SQLite::sqlite_version, need >= v3.8.3 for --jobs > 1
+EOM
+	$opt->{jobs} = '1,1';
+	undef;
+}
+
 1;
